@@ -1,10 +1,9 @@
 package Form::Processor::Model::DBIC;
-use strict;
-use warnings;
+
 use base 'Form::Processor';
 use Carp;
 
-our $VERSION = '0.06';
+our $VERSION = '0.07';
 
 =head1 NAME
 
@@ -21,7 +20,7 @@ Create a Form, subclassed from Form::Processor::Model::DBIC
     use base 'Form::Processor::Model::DBIC';
 
     # Associate this form with a DBIx::Class result class
-    sub object_class { 'MyDB::User' } # Where 'MyDB' is the Catalyst model name
+    sub object_class { 'User' } # Where 'User' is the DBIC source_name 
 
     # Define the fields that this form will operate on
     # Field names must be column or relationship names in your
@@ -102,159 +101,43 @@ active_column).
 For a complex, widget-based TT setup, see the examples directory in the
 L<Catalyst::Plugin::Form::Processor> CPAN download.
  
-Then in a Catalyst controller (with Catalyst::Plugin::Form::Processor):
+Then in a Catalyst controller (with Catalyst::Controller::Form::Processor):
 
     package MyApp::Controller::User;
     use strict;
     use warnings;
-    use base 'Catalyst::Controller';
+    use base 'Catalyst::Controller::Form::Processor';
 
     # Create or edit
     sub edit : Local {
         my ( $self, $c, $user_id ) = @_;
         $c->stash->{template} = 'user/edit.tt'; 
         # Validate and insert/update database. Args = pk, form name
-        return unless $c->update_from_form( $user_id, 'User' );
+        return unless $self->update_from_form( $user_id, 'User' );
         # Form validated.
         $c->stash->{user} = $c->stash->{form}->item;
         $c->res->redirect($c->uri_for('profile'));
     }
 
-With the Catalyst plugin the schema is retrieved from the Catalyst context 
-($c->model()...), but it can also be set by passing in the schema on "new",
-or setting with $form->schema($schema)
+With the Catalyst controller the schema is set from the model_name config
+options, ($c->model($model_name)...), but it can also be set by passing 
+in the schema on "new", or setting with $form->schema($schema). You can
+also set a config values for whether or not to use FillInForm, and
+the form namespace.
 
-In MyApp.pm (the application root controller) config Catalyst::Plugin::F::P:
-
-    BookDB->config->{form} = {
-        no_fillin         => 0,  # Set if you don't want FillInForm
-                                 # to fill in your form values 
-        pre_load_forms    => 1,  # don't set pre_load if using auto fields
-        form_name_space   => 'MyApp::Form',
-        debug             => 1,
-    };
 
 =head1 DESCRIPTION
 
-Form::Processor is a form handling class primarily useful for getting HMTL form
-data into the database. It provides attributes on fields that can be used
-for creating a set of widgets and highly automatic templates, but does
-not actually create the templates themselves. There is a illustrative
-example of a widgetized template setup in the L<Catalyst::Plugin::Form::Processor>
-distribution, and it should be fairly easy to write utilities or scripts 
-to create templates automatically. And cut-and-paste always works...
-
 This DBIC model will save form fields automatically to the database, will
 retrieve selection lists from the database (with type => 'Select' and a 
 fieldname containing a single relationship, or type => 'Multiple' and a
 has_many relationship), and will save the selected values (one value for 
 'Select', multiple values in a mapping table for a 'Multiple' field). 
 
-The 'form' is a Perl subclass of the model class, and in it you define
-your fields (with many possible attributes), and initialization
-and validation routines. Because it's a Perl class, you have a lot of 
-flexibility.
-
-You can, of course, define your own L<Form::Processor::Field> classes to
-create your own field types, and perform specialized validation. And you
-can subclass the methods in Form::Processor::Model::DBIC and 
-Form::Processor.
-
 This package includes a working example using a SQLite database and a
 number of forms. The templates are straightforward and unoptimized to
 make it easier to see what they're doing.
 
-=head1 Combined reference for Form::Processor
-
-Form::Processor has a lot of options and many ways to customize your forms.
-I've collected a list of them here to make them easier to find. More complete
-documentation can be found at L<Form::Processor>, L<Form::Processor::Field>,
-L<Catalyst::Plugin::Form::Processor>, and in the individual field classes.
-
-=head2 Attributes for fields defined in your form:
-
-   name          Field name. Must be the same as database column name or rel
-   type          Field type. From a F::P::Field class: 'Text', 'Select', etc
-   required      Field is required
-   required_message  If this field is required, the message to display on failure 
-   id            Useful for javascript that requires unique id. Set in Field.
-   label         Text label. Not used by F::P, but useful in templates 
-   order         Set the order for fields. Used by sorted_fields, templates. 
-   widget        Used by templates to decide widget usage. Set by field classes.
-   style         Style to use for css formatting. Not used by F::P, for templates.
-   value_format  Sprintf format to use when converting input to value
-   password      Remove from params and do not display in forms. 
-   disabled      HTML hint to not do updates (for templates) Init: 0
-   readonly      HTML hint to make the field readonly (for templates) Init: 0 
-   clear         Don't validate and remove from database
-   noupdate      Don't update this field in the database
-   writeonly     Do not call field class's "format_value" routine. 
-   errors        Errors associated with this field 
-   label_column  Select lists: column to use for labels (default: name)
-   active_column Select lists: which values to list
-   sort_order    Select lists: column to use for sorting (default: label_column)
-   sub_form      The field is made up of a sub-form (only dates at this point)
-   size          Text & select fields. Validated for text.
-   minlength     Text fields. Used in validation
-   range_start   Range start for number fields 
-   range_end     Range end for number fields    
-
-=head2 Field attributes not set in a user form
-
-These attributes are usually accessed in a subroutine or in a template.
-
-   init_value    Initial value from the database (or see init_value_$fieldname) 
-   value         The value of your field. Initially, init_value, then from input.
-   input         Input value from parameter
-   options       Select lists. Sorted array of hashes, keys: "value", "label"
-
-=head2 Other form settings
-
-   dependency    Array of arrays of field names. If one name has a value, all
-                       fields in the list are set to 'required'
-   unique        Arrayref of field names that should be unique in db
-                     or Hashref that also sets message 
-
-=head2 Subroutines for your form (not subclassed)
-
-   object_class             Required for Form::Processor::Model::DBIC (& CDBI)
-   schema                   If you're not using the schema from a Catalyst model
-   options_$fieldname       Provides a list of key value pairs for select lists
-   validate_$fieldname      Validation routine for field 
-   init_value_$fieldname    Overrides initial value for the field
-   cross_validate           For validation after individual fields are validated 
-   active_column            For all select lists in the form
-   init_object              Provide different but similar object to init form 
-                               such as default values (field names must match)
-   field_counter            Increment in templates (see Field & C::P::F::P example)
-   Plus any subroutine you care to write...
-   
-=head2 Methods you might want to subclass from Form::Processor::Model::DBIC
-
-   model_validate    Add additional database type validation
-   update_model      To add additional actions on update
-   guess_field_type  To create better field type assignment for auto forms 
-   many_to_many      If your multiple select list mapping table is not standard
-     
-=head2 Particularly useful in a template
-
-   errors            [% FOREACH error IN form.errors %]
-   error_fields      [% FOREACH field IN form.error_fields %]
-   error_field_names [% FOREACH name IN form.error_field_names %]
-   sorted_fields     [% FOREACH field IN form.sorted_fields %]
-   uuid              subroutine that returns a uuid
-   fif               value="[% form.fif.title %]"
-   params            Same as fif, but password fields aren't stripped
-   
-=head2 L<Form::Processor::Field> subroutines to subclass in a Field class
-
-   validate          Main part of Field subclasses. Generic validation that
-                       applies to all fields of this type.
-   trim_value        If you don't want beginning and ending whitespace trimmed
-   input_to_value    To process the field before storing, after validation
-   validate_field    If you don't want invalid fields cleared, subclass & restore 
-   Add your own field attributes in your custom Field classes.
-    
 =head1 METHODS
 
 =head2 schema
@@ -273,7 +156,7 @@ use Rose::Object::MakeMethods::Generic (
 
     my $validated = $form->update_from_form( $parameter_hash );
 
-This is not the same as the routine called with $c->update_from_form. That
+This is not the same as the routine called with $self->update_from_form. That
 is a Catalyst plugin routine that calls this one. This routine updates or
 creates the object from values in the form.
 
@@ -325,6 +208,15 @@ It currently assumes that any "has_many" relationship name used as a
 field in your form is for a "multiple" select list. This will probably
 change in the future.
 
+This routine allows the use of non-database (non-column, non-relationship) 
+accessors in your result source class. It identifies form fields as 1) column,
+2) relationship, 3) other. Column and other fields are processed and update
+is called on the row. Then relationships are processed.
+
+If the row doesn't exist (no primary key or row object was passed in), then
+a row is created using "create" and the fields identified as columns passed
+in a hashref, followed by "other" fields and relationships.
+
 =cut
 
 sub update_model
@@ -335,88 +227,146 @@ sub update_model
 
    # get a hash of all fields, skipping fields marked 'noupdate'
    my $prefix = $self->name_prefix;
-   my %fields;
-   foreach ($self->fields)
-   {
-      next if $_->noupdate;
-      my $name = $_->name;
-      $name =~ s/^$prefix\.//g if $prefix;
-      $fields{$name} = $_; 
-   }
-   my %data;
+   my %columns;
+   my %multiple_has_many;
+   my %multiple_m2m;
+   my %select;
+   my %other_rel;
+   my %other;
    my $field;
    my $value;
-
-   # First process the normal and has_a columns
-   # as that data is directly stored in the object
-   foreach my $col ( $source->columns )
+   # Save different flavors of fields into hashes for processing
+   foreach $field ( $self->fields )
    {
-      next unless exists $fields{$col};
-      $field = delete $fields{$col};
-
+      next if $field->noupdate;
+      my $name = $field->name;
+      $name =~ s/^$prefix\.//g if $prefix;
       # If the field is flagged "clear" then set to NULL.
       $value = $field->clear ? undef : $field->value;
-
-      if ($item)
+      if ( $source->has_relationship($name) )
       {
-         my $cur = $item->$col;
-         next unless $value || $cur;
-         next if ( ( $value && $cur ) && ( $value eq $cur ) );
-         $item->$col($value);
+         if ( $field->can('multiple') && $field->multiple == 1 ) 
+         {
+            $multiple_has_many{$name} = $value;
+         }
+         # If the table has a column name the same name as the
+         # the "select" relationship, the 'has_columns' will catch it.
+         # This is for Selects with different column name and rel name
+         elsif ($field->can('options'))
+         {
+            $select{$name} = $value;
+         } 
+         else
+         {
+            # for now just remove other relationships because
+            # they aren't handled here, and could be handled in a subclass
+            $other_rel{$name} = $value;
+         }
       }
-      else
+      elsif ( $source->has_column($name) )
       {
-         $data{$col} = $value;
+         $columns{$name} = $value;
+      }
+      elsif ( $field->can('multiple' ) && $field->multiple == 1 )
+      {
+         # didn't have a relationship and is multiple, so must be m2m
+         $multiple_m2m{$name} = $value;
+      }
+      else    # neither a column nor a rel
+      {
+         $other{$name} = $value;
       }
    }
 
+   my $changed = 0;
+   # Handle database columns
    if ($item)
    {
-      $item->update;
+      for my $field_name ( keys %columns )
+      {
+         $value = $columns{$field_name};
+         my $cur = $item->$field_name;
+         next unless $value || $cur;
+         next if ( ( $value && $cur ) && ( $value eq $cur ) );
+         $item->$field_name($value);
+         $changed++;
+      }
       $self->updated_or_created('updated');
    }
-   else
+   else    # create new item
    {
-
-      # create new item
-      $item = $self->resultset->create( \%data );
+      $item = $self->resultset->create( \%columns );
       $self->item($item);
       $self->updated_or_created('created');
    }
 
-   # All non-rel columns and 'has_a' columns have been deleted
-   # from fields hash, so all fields left should be 'has_many' (Multiple)
-   if ( $source->relationships )
+   # Set single select lists with rel different from column
+   for my $field_name ( keys %select )
    {
-      for my $field_name ( keys %fields )
+      my $rel_info = $item->relationship_info($field_name);
+      my ($cond)     = values %{ $rel_info->{cond} };
+      my ($self_col) = $cond =~ m/^self\.(\w+)$/;
+      $item->$self_col( $select{$field_name} );
+      $changed++;
+   }
+
+   # set non-column, non-rel attributes
+   for my $field_name ( keys %other )
+   {
+      next unless $item->can($field_name);
+      $item->$field_name( $other{$field_name} );
+      $changed++;
+   }
+   # update db
+   $item->update if $changed > 0;
+
+   # process Multiple field 'has_many' relationships
+   for my $field_name ( keys %multiple_has_many )
+   {
+      # This is a has_many/many_to_many relationship
+      my ( $self_rel, $self_col, $foreign_rel, $foreign_col, $m2m_rel ) =
+         $self->many_to_many($field_name);
+      $value = $multiple_has_many{$field_name};
+      my %keep;
+      %keep = map { $_ => 1 } ref $value ? @$value : ($value)
+         if defined $value;
+      if ( $self->updated_or_created eq 'updated' )
       {
-         next
-           unless ( ( $source->has_relationship($field_name) )
-            && ( $source->relationship_info($field_name)->{attrs}->{accessor} eq 'multi' ) );
-
-         # This is a has_many/many_to_many relationship
-         my ( $self_rel, $self_col, $foreign_rel, $foreign_col, $m2m_rel ) =
-           $self->many_to_many($field_name);
-
-         $field = delete $fields{$field_name};
-         $value = $field->value;
-         my %keep;
-         %keep = map { $_ => 1 } ref $value ? @$value : ($value)
-           if defined $value;
-
-         if ( $self->updated_or_created eq 'updated' )
+         for ( $item->$field_name->all )
          {
-            for ( $item->$field_name->all )
-            {
+            # delete old selections
+            $_->delete unless delete $keep{ $_->$foreign_col };
+          }
+      }
 
-               # delete old selections
-               $_->delete unless delete $keep{ $_->$foreign_col };
-            }
+      # Add new related
+      $item->create_related( $field_name, { $foreign_col => $_ } ) for keys %keep;
+   } 
+   # process Multiple field 'many_to_many' relationships
+   for my $field_name ( keys %multiple_m2m )
+   {
+      $value = $multiple_m2m{$field_name};
+      my %keep;
+      %keep = map { $_ => 1 } ref $value ? @$value : ($value)
+         if defined $value;
+      my $meth;
+      my $row;
+      if ( $self->updated_or_created eq 'updated' )
+      {
+         foreach $row ( $item->$field_name->all )
+         {
+            $meth = 'remove_from_' . $field_name;
+            $item->$meth( $row ) 
+               unless delete $keep{ $row->id };
          }
-
-         # Add new related
-         $item->create_related( $field_name, { $foreign_col => $_ } ) for keys %keep;
-      }    # end of key processing loop
+      }
+      my $source_name = $item->$field_name->result_source->source_name;
+      foreach my $id ( keys %keep )
+      {
+         $row = $self->schema->resultset($source_name)->find($id);
+         $meth = 'add_to_' . $field_name;
+         $item->$meth( $row );
+      }
    }
 
    # Save item in form object
@@ -425,6 +375,7 @@ sub update_model
    return $item;
 }
 
+
 =head2 guess_field_type
 
 This subroutine is only called for "auto" fields, defined like:
@@ -445,8 +396,7 @@ otherwise:
     Text            - otherwise
 
 Subclass this method to do your own field type assignment based
-on column types. Don't use "pre_load_forms" if using auto fields.
-The DBIx::Class schema isn't available yet.
+on column types. This routine returns either an array or type string. 
 
 =cut
 
@@ -461,17 +411,16 @@ sub guess_field_type
    # Is it a direct has_a relationship?
    if (
       $source->has_relationship($column)
-      && (  $source->relationship_info($column)->{attrs}->{accessor} eq 'single' ||
-            $source->relationship_info($column)->{attrs}->{accessor} eq 'filter' )
-     )
+      && (  $source->relationship_info($column)->{attrs}->{accessor} eq 'single'
+         || $source->relationship_info($column)->{attrs}->{accessor} eq 'filter' )
+      )
    {
       my $f_class = $source->related_class($column);
       @return =
-        $f_class->isa('DateTime')
-        ? ('DateTimeDMYHM')
-        : ('Select');
+         $f_class->isa('DateTime')
+         ? ('DateTimeDMYHM')
+         : ('Select');
    }
-
    # Else is it has_many?
    elsif ( $source->has_relationship($column)
       && $source->relationship_info($column)->{attrs}->{accessor} eq 'multi' )
@@ -524,41 +473,41 @@ sub lookup_options
    my ( $self, $field ) = @_;
 
    my $field_name = $field->name;
-   my $prefix = $self->name_prefix;
+   my $prefix     = $self->name_prefix;
    $field_name =~ s/^$prefix\.//g if $prefix;
 
    # if this field doesn't refer to a foreign key, return
-   my $rel_info = $self->source->relationship_info($field_name);
    my $f_class;
-   $f_class = $self->source->related_class($field_name)
-     if $self->source->has_relationship($field_name);
-   return unless $f_class;
-
-   # This field refers to foreign table, so continue.
-
-   my $source = $self->schema->source($f_class);
-
-   # this is a bit messy, but leaving here for now since
-   # it will probably get even more complicated in the future...
-   if ( $field->type eq 'Multiple' ||
-      ( $field->type eq 'Auto' && $rel_info->{attrs}{accessor} eq 'multi' ) )
+   my $source;
+   if ($self->source->has_relationship($field_name) )
    {
+      $f_class = $self->source->related_class($field_name);
+      $source = $self->schema->source($f_class);
 
-      # This is a 'has_many' relationship with a mapping table
-      my ( $self_rel, $self_col, $foreign_rel, $foreign_col ) =
-        $self->many_to_many( $field_name );
-
-      $f_class = $source->related_class($foreign_rel);
-      $source  = $source->related_source($foreign_rel);
+      my $rel_info = $self->source->relationship_info($field_name);
+      if ( $field->type eq 'Multiple'
+         || ( $field->type eq 'Auto' && $rel_info->{attrs}{accessor} eq 'multi' ) )
+      {
+         # This is a 'has_many' relationship with a mapping table
+         my ( $self_rel, $self_col, $foreign_rel, $foreign_col ) =
+            $self->many_to_many($field_name);
+         $source  = $source->related_source($foreign_rel);
+      }
    }
+   elsif ($self->resultset->new_result({})->can("add_to_$field_name") )
+   {
+      # Multiple field with many_to_many relationship
+      $source = $self->resultset->new_result({})->$field_name->result_source;
+   }
+   return unless $source; 
 
    my $label_column = $field->label_column;
    return unless $source->has_column($label_column);
 
    my $active_col =
-       $self->can('active_column')
-     ? $self->active_column
-     : $field->active_column;
+        $self->can('active_column')
+      ? $self->active_column
+      : $field->active_column;
 
    $active_col = '' unless $source->has_column($active_col);
    my $sort_col = $field->sort_order;
@@ -574,25 +523,26 @@ sub lookup_options
 
       # But also include any existing non-active
       push @or, ( "$primary_key" => $field->init_value )
-        if $self->item && defined $field->init_value;
+         if $self->item && defined $field->init_value;
       $criteria->{'-or'} = \@or;
    }
 
    # get an array of row objects
-   my @rows = $self->schema->resultset($source->source_name)->search( $criteria, { order_by => $sort_col } )->all;
+   my @rows =
+      $self->schema->resultset( $source->source_name )
+      ->search( $criteria, { order_by => $sort_col } )->all;
 
    return [
       map {
          my $label = $_->$label_column;
          $_->id, $active_col && !$_->$active_col ? "[ $label ]" : "$label"
-        } @rows
+         } @rows
    ];
-
 }
 
 =head2 init_value
 
-This method return's a field's value (for $field->value) with
+This method returns a field's value (for $field->value) with
 either a scalar or an array ref from the object stored in $form->item.
 
 This method is not called if a method "init_value_$field_name" is found 
@@ -605,42 +555,63 @@ sub init_value
 {
    my ( $self, $field, $item ) = @_;
 
-   my $column = $field->name;
+   my $name = $field->name;
    my $prefix = $self->name_prefix;
-   $column =~ s/$prefix\.//g if $prefix;
-
+   $name =~ s/$prefix\.//g if $prefix;
    $item ||= $self->item;
-   return                  unless $item;
-   return $item->{$column} unless $item->isa('DBIx::Class');
-   return                  unless $item->can($column);
-   return                  unless defined $item->$column;
+   return unless $item;
+   return $item->{$name} if ref($item) eq 'HASH';
+   return unless $item->isa('DBIx::Class') && $item->can($name);
+   return unless defined $item->$name;
 
    my $source = $self->source;
-
-   if ( !$source->has_relationship($column) )    # We already know it "can" $column
+   if ( $source->has_relationship($name) )
    {
-      return ($item->$column);
+      if ( $field->can('multiple') && $field->multiple == 1 ) 
+      {
+         # has_many Multiple field
+         my ( undef, undef, undef, $foreign_col ) = $self->many_to_many($name);
+         my @rows = $item->search_related($name)->all;
+         my @values = map { $_->$foreign_col } @rows;
+         return @values;
+      }
+      elsif ($field->can('options'))
+      {
+         return $item->$name->id; 
+      } 
+      else # some other relationship (unsupported)
+      {
+         my $rel_info = $source->relationship_info($name);
+         if ( $rel_info->{attrs}->{accessor} eq 'single' ||
+              $rel_info->{attrs}->{accessor} eq 'filter' )
+         {
+            return $item->$name->get_inflated_columns; 
+         }
+         else # multi relationship (unsupported)
+         {
+            my $rs = $item->$name;
+            $rs->result_class('DBIx::Class::ResultClass::HashRefInflator'); 
+            return $rs->all;
+         }
+      }
    }
-   elsif ( $source->relationship_info($column)->{attrs}->{accessor} eq 'single' ||
-           $source->relationship_info($column)->{attrs}->{accessor} eq 'filter' )
+   elsif ( $source->has_column($name) )
    {
-      return ($item->$column->id);
+      return $item->$name; 
    }
-
-   # this is a 'has_many' relationship and we must return an array of row objects
-   elsif ( $source->relationship_info($column)->{attrs}->{accessor} eq 'multi' )
+   elsif ( $field->can('multiple' ) && $field->multiple == 1 )
    {
-      my ( $self_rel, $self_col, $foreign_rel, $foreign_col ) = $self->many_to_many($column);
-      my @rows = $item->search_related($column)->all;
-      my @values = map { $_->$foreign_col } @rows;
+      my @rows = $item->$name->all;
+      my @values = map { $_->id } @rows;
       return @values;
    }
-   else
+   else    # neither a column nor a rel
    {
-      $field->add_error("Could not identify column or relationship.");
+      return $item->$name;
    }
 }
 
+
 =head2 validate_unique
 
 For fields that are marked "unique", checks the database for uniqueness.
@@ -659,34 +630,29 @@ sub validate_unique
 {
    my ($self) = @_;
 
-   my $unique      = $self->profile->{unique} || return 1;
+   my $unique      = $self->profile->{unique};
    my $item        = $self->item;
    my $rs          = $self->resultset;
    my $found_error = 0;
-
    my @unique_fields;
    my $error_message;
-   if ( ref($unique) eq 'ARRAY' ) 
+   if ( ref($unique) eq 'ARRAY' )
    {
       @unique_fields = @$unique;
       $error_message = 'Value must be unique in the database';
-   } 
-   elsif ( ref($unique) eq 'HASH' ) 
+   }
+   if ( ref($unique) eq 'HASH' )
    {
-       @unique_fields = keys %$unique;
-   } 
-   else 
-   {
-      return;
+      @unique_fields = keys %$unique;
    }
 
+   return 1 unless @unique_fields;
    for my $field ( map { $self->field($_) } @unique_fields )
    {
-
       next if $field->errors;
       my $value = $field->value;
       next unless defined $value;
-      my $name = $field->name;
+      my $name   = $field->name;
       my $prefix = $self->name_prefix;
       $name =~ s/^$prefix\.//g if $prefix;
 
@@ -696,11 +662,13 @@ sub validate_unique
       # not found, this one is unique
       next if $count < 1;
       # found this value, but it's the same row we're updating
-      next if $count == 1 
-           && $self->item_id 
-           && $self->item_id == $rs->search({$name => $value})->first->id; 
-      $field->add_error($error_message
-          || $self->profile->{'unique'}->{$name});
+      next
+         if $count == 1
+            && $self->item_id
+            && $self->item_id == $rs->search( { $name => $value } )->first->id;
+      my $field_error = $field->unique_message || $error_message || 
+            $self->profile->{'unique'}->{$name};
+      $field->add_error( $field_error );
       $found_error++;
    }
 
@@ -722,17 +690,21 @@ It then does:
 It also validates that the item id matches /^\d+$/.  Override this method
 in your form class (or form base class) if your ids do not match that pattern.
 
+If a database row for the item_id is not found, item_id will be set to undef.
+
 =cut
 
 sub init_item
 {
    my $self = shift;
+
    my $item_id = $self->item_id or return;
    return unless $item_id =~ /^\d+$/;
-   return $self->resultset->find($item_id);
+   my $item = $self->resultset->find($item_id);
+   $self->item_id(undef) unless $item;
+   return $item;
 }
 
-
 =head2 init_schema
 
 Initializes the DBIx::Class schema. User may override. Non-Catalyst
@@ -744,7 +716,7 @@ $my_form_class->new(item_id => $id, schema => $schema)
 sub init_schema
 {
    my $self = shift;
-   return if defined $self->{schema};
+   return if exists $self->{schema};
    if ( my $c = $self->user_data->{context} ) 
    {
        # starts out <model>::<source_name>
@@ -753,8 +725,13 @@ sub init_schema
        $self->source_name( $c->model( $self->object_class )->result_source->source_name );
        return $schema;
    }
-   die "Unable to find DBIx::Class schema";
+   die "Schema must be defined for Form::Processor::Model::DBIC";
+}
 
+sub init_source_name
+{
+   my $self = shift;
+   return $self->object_class;
 }
 
 =head2 source
@@ -766,7 +743,7 @@ Returns a DBIx::Class::ResultSource object for this Result Class.
 sub source
 {
    my ( $self, $f_class ) = @_;
-   return $self->schema->source($self->source_name || $self->object_class);
+   return $self->schema->source( $self->source_name || $self->object_class );
 }
 
 =head2 resultset
@@ -780,7 +757,7 @@ a relationship.
 sub resultset
 {
    my ( $self, $f_class ) = @_;
-   return $self->schema->resultset($self->source_name || $self->object_class);
+   return $self->schema->resultset( $self->source_name || $self->object_class );
 }
 
 =head2 many_to_many
@@ -867,22 +844,18 @@ sub _build_fields
    }
 }
 
-=head1 SUPPORT
-
-The author can be contacted through the L<Catalyst> or L<DBIx::Class> mailing 
-lists or IRC channels (gshank).
 
 =head1 SEE ALSO
 
 L<Form::Processor>
 L<Form::Processor::Field>
 L<Form::Processor::Model::CDBI>
-L<Catalyst::Plugin::Form::Processor>
+L<Catalyst::Controller:Form::Processor>
 L<Rose::Object>
 
 =head1 AUTHOR
 
-Gerda Shank
+Gerda Shank, gshank@cpan.org
 
 =head1 CONTRIBUTORS
 
@@ -896,4 +869,3 @@ the same terms as Perl itself.
 =cut
 
 1;
-
